%%
%
% Gaussian Random Paths
%
ccc

%%
ccc
global click_pos flag key_pressed
click_pos = [0 0]; key_pressed = ''; flag = 'plot';
fig = figure(1);
set(fig, 'WindowButtonDownFcn', @buttonDownCallback,'KeyPressFcn', @keyDownListener ...
    , 'Position', [200 200 800 500], 'NumberTitle', 'off', 'Name', 'Gaussian Random Paths');
ppos = zeros(1E3, 2); nrppos = 0; emsec = 0; nrPath = 0;
while ~isequal(flag, 'terminate') && ishandle(fig)
    % Click handler 
    switch flag 
        case 'normal' % left click
            nrppos = nrppos + 1; 
            ppos(nrppos, :) = click_pos;
            flag = 'update';
        case 'alt'
            if nrppos > 0, nrppos = nrppos -1; end;
            flag = 'update';
    end
    % Find connecting GRP
    if isequal(flag, 'update') 
        xy = ppos(1:nrppos, :);
        if nrppos >= 2
            ts = get_ts(xy); xs = xy(:, 1); ys = xy(:, 2);
            testInputs = linspace(0, ts(end), 50)';
            kfun   = @(X1, X2)(kernel_se(X1, X2, struct('g2', 5^2, 'A', 1/(5)^2, 'w2', 0, 'mzflag', 0)));
            nrPath = 1000;
            iclk = clock;
            [xPaths, xMeanPath] = grp(ts, xs, testInputs, kfun, nrPath);
            [yPaths, yMeanPath] = grp(ts, ys, testInputs, kfun, nrPath);
            emsec = etime(clock, iclk)*1000;
        end
        flag = 'plot';
    end
    % Plot
    if isequal(flag, 'plot') 
        clf; hold on; 
        if nrppos >= 2
            colors = copper(nrPath);
            for i = 1:nrPath
                plot(xPaths(i, :), yPaths(i, :), '-', 'Color', colors(i, :));
            end
            plot(xMeanPath, yMeanPath, 'b--', 'LineWidth', 3);
        end
        if nrppos > 0, hp = plot(xy(:, 1), xy(:, 2), 'bo', 'MarkerSize', 11, 'LineWidth', 2); end
        xlabel('X', 'FontSize', 13); ylabel('Y', 'FontSize', 13);
        axis equal; axis([0 10 0 5]); grid on; title(sprintf('Sampled %d paths in %.1fms', nrPath, emsec));
        drawnow;
        flag = 'waiting';
    end
    if isequal(key_pressed, 'q')
        flag = 'terminate';
    else
        key_pressed = '';
    end
    pause(1E-10);
end
fprintf(2, 'Terminated. \n');



