%%
%
% Nonlinear Regression using Kernel Methods
% 
ccc

%%
ccc
global click_pos flag key_pressed
click_pos = [0 0]; key_pressed = ''; flag = 'plot';

xtest = linspace(0, 10, 100)';
ytest = zeros(100, 1);
xu    = linspace(0, 10, 5)'; % Inducing points
kfun  = @(X1, X2)(kernel_se(X1, X2, struct('g2', 1, 'A', 1/(2)^2, 'w2', 0E-6, 'mzflag', 0)));
fig = figure(1);
set(fig, 'WindowButtonDownFcn', @buttonDownCallback,'KeyPressFcn', @keyDownListener ...
    , 'Position', [200 200 800 500], 'NumberTitle', 'off', 'Name', 'Kernel Regression');
ppos = zeros(1E3, 2); nrppos = 0; 
while ~isequal(flag, 'terminate') && ishandle(fig)
    % Click handler 
    switch flag 
        case 'normal' % left click
            nrppos = nrppos + 1; 
            ppos(nrppos, :) = click_pos;
            flag = 'update';
        case 'alt'
            if nrppos > 0, nrppos = nrppos -1; end;
            flag = 'update';
    end
    % Find Nonlinear Function
    if isequal(flag, 'update') 
        X        = ppos(1:nrppos, 1); 
        Y        = ppos(1:nrppos, 2); 
        Kxz      = kfun(X, xu);
        alphabar = (Kxz'*Kxz + 1E-4*eye(length(xu)))\Kxz'*Y;
        Ktest    = kfun(xtest, xu); 
        ytest    = Ktest*alphabar;
        flag = 'plot';
    end
    % Plot
    if isequal(flag, 'plot') 
        clf; hold on; 
        hp = plot(ppos(1:nrppos, 1), ppos(1:nrppos, 2), 'bo', 'MarkerSize', 15, 'LineWidth', 2);
        plot(xtest, ytest, 'k', 'LineWidth', 3);
        xlabel('x (input)', 'FontSize', 13); ylabel('f(x) (output)', 'FontSize', 13);
        axis equal; axis([0 10 -3 3]); grid on;
        title(sprintf('Kernel Regression with %d Positions', nrppos));
        drawnow;
        flag = 'waiting';
    end
    if isequal(key_pressed, 'q')
        flag = 'terminate';
    else
        key_pressed = '';
    end
    pause(1E-10);
end
fprintf(2, 'Terminated. \n');


%%

